<?php

/*
**
** Definitions and Declarations
**
*/
define(DIMENSION, 400);           // half the size of the canvas
define(OFFSET, 50);               // axis offset from canvas margins
define(LABELX, 50);               // x coord of function label
define(LABELY, 50);               // y coord of function label

$aPoints = array(x => array(), y => array());   // array to contain all the points of the equation

$sFormula = "";                   // Formula from user input
$sError = "";                     // error message if there was an error

// equation variables
$iMultiplier = -1;
$iPower = -1;
$cOp = ".";
$iConstant = -1;

// parsing test arrays
$aDigits = array( 0 => "0",
                  1 => "1",
                  2 => "2",
                  3 => "3",
                  4 => "4",
                  5 => "5",
                  6 => "6",
                  7 => "7",
                  8 => "8",
                  9 => "9");
$aOps = array( "+" => "+", "-" => "-", "*" => "*", "/" => "/" );
$iIndex = 0;

/*
**
** form handler
** Jumps: to end
**
*/
// check if we are called from the form
if (!array_key_exists("fn", $_REQUEST))
   goto end;

// prepare for parsing
$sFormula = $_REQUEST['fn'];
$aFormula = str_split($sFormula);

/*
** NextChar()
**
** advances the pointer into our function to the next character while checking for the end of the equation string
**
*/
function NextChar() {
global $aFormula, $iIndex,$sError;

   $iIndex++;
   if ($iIndex >= count($aFormula)) {
      echo "Incomplete formula: ".$sFormula,". Bailing! ".$iIndex." ".count($aFormula)." ";
      return false;
   }
   return true;
}


/*
**
** Parser code for our equation string
** Uses: NextChar
** Jumps: to end and to process
**
*/
// parse the y
if ($aFormula[$iIndex] != "y") {
   $sError = "Formula must start with a y variable.";
   goto end;
} else if (!NextChar()) goto end;

// parse the possible space
if ($aFormula[$iIndex] == " ") if (!NextChar()) goto end;

// parse the equals sign
if ($aFormula[$iIndex] != "=") {
   $sError = "Right hand side and left hand side of formula must be separated with an equals (=) operator.";
   goto end;
} else if (!NextChar()) goto end;

// parse the possible space
if ($aFormula[$iIndex] == " ") if (!NextChar()) goto end;
  
// parse number or x - note no space allowed between numbers and spaces
if ($aFormula[$iIndex] == "x") {
   if (!NextChar()) goto end;
} else if (array_key_exists($aFormula[$iIndex], $aDigits)) {
   // handle multi digit numbers
   while (array_key_exists($aFormula[$iIndex], $aDigits)) {
      if ($iMultiplier == -1) 
         $iMultiplier = $aFormula[$iIndex];
      else 
         $iMultiplier = $iMultiplier * 10 + $aFormula[$iIndex];
      if (!NextChar()) goto end;
   }
   if ($aFormula[$iIndex] != "x") {
      $sError = "Multiplier numbers must be followed by the x variable.";
      goto end;
   }
   // process the x variable
   if (!NextChar()) goto end;
} else {
   $sError = "Right hand side of formula must start with a number or x variable.";
   goto end;
}

// parse possible power
if ($aFormula[$iIndex] == "^") {
   if (!NextChar()) goto end;
   // handle multi digit numbers
   while (array_key_exists($aFormula[$iIndex], $aDigits)) {
      if ($iPower == -1) 
         $iPower = $aFormula[$iIndex];
      else 
         $iPower = $iPower * 10 + $aFormula[$iIndex];
      if (!NextChar()) goto end;
   }
}

// parse the possible space
if ($aFormula[$iIndex] == " ") if (!NextChar()) goto end;

// check for the end of the formula
if ($iIndex >= count($aFormula)) goto process;

// parse operator
if (array_key_exists($aFormula[$iIndex], $aOps)) {
   $cOp = $aOps[$aFormula[$iIndex]];
   if (!NextChar()) goto end;
} else {
   $sError = "Unknown operator: ".$aFormula[$iIndex];
   goto end;
}

// parse the possible space
if ($aFormula[$iIndex] == " ") if (!NextChar()) goto end;

// Parse the possible constant
if (array_key_exists($aFormula[$iIndex], $aDigits)) {
   // handle multi digit numbers
   while (array_key_exists($aFormula[$iIndex], $aDigits)) {
      if ($iConstant == -1) 
         $iConstant = $aFormula[$iIndex];
      else 
         $iConstant = $iConstant * 10 + $aFormula[$iIndex];
      $iIndex++;
      if ($iIndex >= count($aFormula))    
         break;
   }
}


/*
**
** Generates the points to graph from our equation
** Jumps: from process
**
*/
process:
// walk the arrays generating the result in the y array
for ($i = 0; $i < (DIMENSION - OFFSET); $i++) {

    // assign x value and initial y value ie same as y = x
    $aPoints["x"][$i] = $i;    
    $aPoints["y"][$i] = $i;
    if ($i != 0) {
       $aPoints["x"][$i * -1] = $i * -1;
       $aPoints["y"][$i * -1] = $i * -1;
    }

    // apply the power to y if there is one
    if ($iPower != -1) {
       $aPoints["y"][$i] = pow($aPoints["y"][$i], $iPower);
        if ($i != 0) $aPoints["y"][$i * -1] = pow($aPoints["y"][$i * -1], $iPower);
    }

    // apply the multiplier to y if there is one
    if ($iMultiplier != -1) {
       $aPoints["y"][$i] = $aPoints["y"][$i] * $iMultiplier;
        if ($i != 0) $aPoints["y"][$i * -1] = $aPoints["y"][$i * -1] * $iMultiplier;
    }

    // apply the constant to y is there is one   
    switch($cOp) {
       case "+" :
             $aPoints["y"][$i] = $aPoints["y"][$i] + $iConstant;
             if ($i != 0) $aPoints["y"][$i * -1] = $aPoints["y"][$i * -1] + $iConstant;
             break;
       case "-" :
             $aPoints["y"][$i] = $aPoints["y"][$i] - $iConstant;
              if ($i != 0) $aPoints["y"][$i * -1] = $aPoints["y"][$i * -1] - $iConstant;
             break;
       case "*" :
             $aPoints["y"][$i] = $aPoints["y"][$i] * $iConstant;
             if ($i != 0) $aPoints["y"][$i * -1] = $aPoints["y"][$i * -1] * $iConstant;
             break;
       case "+" :
             $aPoints["y"][$i] = $aPoints["y"][$i] / $iConstant;
             if ($i != 0) $aPoints["y"][$i * -1] = $aPoints["y"][$i * -1] / $iConstant;
             break;
    }
}
end:
if ($sError != "")
    echo $sError."<br>";;
?>

<!DOCTYPE HTML>
   <html>            
       <head>
          <title>Equation Grapher</title>          
       </head>
       <body>
          <table>
             <tr>
               <td>   
                   <form action="<?php echo $_SERVER['HTTP_REFERER'];?>" method="post">
                      All equations need to be of the form:<br>
                      &nbsp;&nbsp;&nbsp;y = [&lt;number&gt;]x[^&lt;number&gt;][&lt;operator&gt;&lt;number&gt;]<br>
                      eg: 
                      <ul>
                          <li> y = 5x + 9</li>
                          <li> y = x^2 - 10 </l1>
                      </ul><br>

                      Equation: <input type="text" name="fn"><p />
                      <input type="submit" value="Submit"><br>
                   </form>                    
               </td>
               <td>
                   <canvas id="myCanvas" width="<?php echo DIMENSION * 2 ?>" height="<?php echo DIMENSION * 2 ?>" style="border:1px solid #000000;">
                   </canvas>
                   <script>
                      var c = document.getElementById("myCanvas");
                      var ctx = c.getContext("2d");
                      ctx.font = "25px Arial";
                      ctx.fillText(<?php echo '"'.$sFormula.'", '.LABELX.", ".LABELY; ?>);
                      ctx.fillText("y",<?php echo DIMENSION - 3; ?>, <?php echo OFFSET - 10; ?>);
                      ctx.fillText("x",<?php echo OFFSET - 15; ?>,<?php echo DIMENSION + 5; ?>);
                      ctx.beginPath();
                      ctx.strokeStyle = 'black';
                      ctx.moveTo(<?php echo DIMENSION; ?>,<?php echo OFFSET; ?>);
                      ctx.lineTo(<?php echo DIMENSION; ?>,<?php echo DIMENSION * 2 - OFFSET; ?>);
                      ctx.stroke();
                      ctx.moveTo(<?php echo OFFSET; ?>,<?php echo DIMENSION; ?>);
                      ctx.lineTo(<?php echo DIMENSION * 2 - OFFSET; ?>,<?php echo DIMENSION; ?>);
                      ctx.stroke();
                      ctx.beginPath();
                      ctx.strokeStyle = 'red';
                      <?php
                           // walk the arrays generating the result in the y array
                         if (count($aPoints["x"]) > 0)
                            for ($i = 0; $i < (DIMENSION - OFFSET - 1); $i++) {
                               $iThisXP = DIMENSION + $aPoints["x"][$i];
                               $iThisYP = DIMENSION - $aPoints["y"][$i];
                               $iThisXN = DIMENSION + $aPoints["x"][$i * -1];
                               $iThisYN = DIMENSION - $aPoints["y"][$i * -1];
                               $iNextXP = DIMENSION + $aPoints["x"][$i + 1];
                               $iNextYP = DIMENSION - $aPoints["y"][$i + 1];
                               $iNextXN = DIMENSION + $aPoints["x"][($i + 1) * -1];
                               $iNextYN = DIMENSION - $aPoints["y"][($i + 1) * -1];
                               if (($iThisXP <= (DIMENSION * 2) - OFFSET) &&
                                   ($iThisXP >= OFFSET) &&
                                   ($iThisYP <= (DIMENSION * 2) - OFFSET) &&
                                   ($iThisYP >= OFFSET) &&
                                   ($iNextXP <= (DIMENSION * 2) - OFFSET) &&
                                   ($iNextXP >= OFFSET) &&
                                   ($iNextYP <= (DIMENSION * 2) - OFFSET) &&
                                   ($iNextYP >= OFFSET)) {
                                      echo "                      ctx.moveTo($iThisXP,$iThisYP);\n";
                                      echo "                      ctx.lineTo($iNextXP,$iNextYP);\n";
                                      echo "                      ctx.stroke();\n";
                                      echo "                      ctx.moveTo($iThisXN,$iThisYN);\n";
                                      echo "                      ctx.lineTo($iNextXN,$iNextYN);\n";
                                      echo "                      ctx.stroke();\n";
                              }
                          }
                      ?>
                  </script>
               </td>
             </tr>
           </table>
       </body>
   </html>
